@@ -6,6 +6,7 @@
 
 /*** GnomeICU header files ***/
 #include "common.h"
+#include "gtkspell.h"
 
 /*** Toplevel header files ***/
 #include <stdio.h>
@@ -26,23 +27,14 @@
 #include <assert.h>
 #include <sys/wait.h>
 #include <signal.h>
+#ifdef GNOME_ICON
+#include <libgnomeui/gnome-window-icon.h>
+#endif
 #ifdef HAVE_SOCKS5
 #define SOCKS
 #include <socks.h>
 #endif
 
-/*** XPM files ***/
-#ifdef USE_STATUS_MENU
-#include "xpms/gnomeicu-away.xpm"
-#include "xpms/gnomeicu-na.xpm"
-#include "xpms/gnomeicu-occ.xpm"
-#include "xpms/gnomeicu-dnd.xpm"
-#include "xpms/gnomeicu-ffc.xpm"
-#include "xpms/gnomeicu-inv.xpm"
-#include "xpms/gnomeicu-online.xpm"
-#include "xpms/gnomeicu-offline.xpm"
-#endif
-
 gboolean Quit = FALSE;
 WORD seq_num = 1;  /* current sequence number */
 DWORD our_ip = 0x7f000001; /* localhost for some reason */
@@ -122,6 +114,8 @@ gint autohide_time = 0;
 gint autohide_timeout_tag = 0; /* timeout tag, 0 means it's inactive */
 gboolean autohide_cbs_are_connected = FALSE;
 
+gint autoaway_timeout_tag = 0;
+
 gint min_tcp_port = 4000;
 gint max_tcp_port = 5000;
 
@@ -130,6 +124,7 @@ gboolean Connected = FALSE;
 USER_INFO_PTR our_info;
 
 _toggles *toggles;
+_programs *programs;
 
 gboolean is_new_user = FALSE;
 
@@ -174,8 +169,9 @@ void source_drag_data_get (GtkWidget *widget, GdkDragContext *context,
                            guint time, gpointer data);
 void change_userwin( GtkWidget *widget, GtkNotebookPage *page, int num,
                      gpointer data );
+void window_save_size( GtkWidget *window, GtkAllocation *req,
+			      gpointer data );
 
-                                                                                                                                       
 /*** Global functions ***/
 void create_tcp_line( void )
 {
@@ -217,6 +213,7 @@ void create_tcp_line( void )
 
 		g_free( our_info->ip );
 
+		/* FIXME: this should be freed sometime */
 		our_info->ip = g_strdup_printf( "%u.%u.%u.%u",
 		         (BYTE) (our_ip >> 24),
 		         (BYTE) (our_ip >> 16),
@@ -241,7 +238,7 @@ void ready_set( void )
 	GtkWidget *pixmap;
 
 #ifdef TRACE_FUNCTION
-	g_print( "ready_set(%08X)\n", Current_Status );
+	g_print( "ready_set(%08x)\n", Current_Status );
 #endif
 
 	if ( Current_Status == STATUS_OFFLINE )
@@ -297,6 +294,41 @@ void ready_set( void )
 
 	gtk_container_add( GTK_CONTAINER( status_button ), status_im );
 	gtk_widget_show( status_im );
+
+	gtk_pixmap_set( GTK_PIXMAP( eyes_still ),
+                        still_eyes_pixmap, still_eyes_bitmap );
+}
+
+void create_animator( GtkWidget *hbox_param )
+{
+    GtkWidget *hbox;
+    gchar *str;
+
+    eyes_animated = MainData->animator;
+
+    if( GTK_IS_WIDGET( hbox_param ) )
+        hbox = hbox_param;
+    else
+        hbox = gtk_object_get_data( GTK_OBJECT( eyes_animated ), "parent_hbox" );
+
+    if( GTK_IS_WIDGET( eyes_animated ) )
+    {
+        gtk_widget_destroy( eyes_animated );
+    }
+
+    /* FIXME: this is never freed */
+    eyes_animated = gnome_animator_new_with_size( 29, 14 );
+    gtk_object_set_data( GTK_OBJECT( eyes_animated ), "parent_hbox",
+                         hbox );
+    MainData->animator = eyes_animated;
+
+    gtk_box_pack_end( GTK_BOX( hbox ), eyes_animated, FALSE, FALSE, 0 );
+
+    str = make_file_path( "gnomeicu-animation.png" );
+
+    gnome_animator_append_frames_from_file( GNOME_ANIMATOR( eyes_animated ),
+                                            str, 0, 0, 100, 29 );
+    g_free( str );
 }
 
 void toggle_log_window( GtkWidget *widget, gpointer data )
@@ -309,6 +341,17 @@ void toggle_log_window( GtkWidget *widget, gpointer data )
 		gtk_widget_show( log_window );
 }
 
+void window_save_size( GtkWidget *window, GtkAllocation *req,
+			      gpointer data )
+{
+    if ((WindowHeight != req->height) || (WindowWidth) != req->width) {
+	WindowHeight = req->height;
+	WindowWidth = req->width;
+	gtk_window_set_default_size (GTK_WINDOW (window),
+				     WindowWidth, WindowHeight );
+    }
+}
+
 void init( void )
 {
 #ifdef TRACE_FUNCTION
@@ -320,17 +363,24 @@ void init( void )
 
 	Done_Login = TRUE;
 
-	gtk_timeout_add( 120000, (GtkFunction) stay_connected, NULL );
+	gtk_timeout_add( 60000, (GtkFunction) stay_connected, NULL );
 	gtk_timeout_add( 500, (GtkFunction) flash_messages, NULL );
-#ifdef USE_AUTOAWAY
-	gtk_timeout_add( 1000, (GtkFunction) auto_away, NULL );
-#endif
+	if (toggles->use_autoaway) autoaway_timeout_tag = gtk_timeout_add( 1000, (GtkFunction) auto_away, NULL );
 
 	been_thru_init = TRUE;
 }
 
 /*** Local functions ***/
 
+/* Broken pipe catching function */
+/* prototype to make gcc happy */
+void broken_pipe (int);
+void broken_pipe (int sig_num)
+{
+  fprintf(stderr, _("gnomeicu: broken pipe (saving configuration)"));
+  icq_quit (NULL, NULL);
+}
+
 /*** MAIN function ***/
 int main( int argc, char *argv[] )
 {
@@ -344,7 +394,7 @@ int main( int argc, char *argv[] )
 	static struct poptOption arguments[] =
 	{
 		{"uin", 'u', POPT_ARG_STRING, &cfnp, 0, N_("Use the defined UIN"), N_("UIN")},
-		{NULL, 'a', POPT_ARG_NONE, 0, 0, N_("Startup without applet support"), NULL},
+		{"noapplet", 'a', POPT_ARG_NONE, 0, 0, N_("Startup without applet support"), NULL},
 		{NULL, 0, 0, NULL, 0, NULL, NULL}
 	};
 
@@ -373,54 +423,72 @@ int main( int argc, char *argv[] )
 
 	GSList *contact;
 
-	gchar *str;
-
 	gboolean have_tcp_port = FALSE;
 
 #ifdef TRACE_FUNCTION
 	g_print( "main\n" );
 #endif
-
-	bindtextdomain(PACKAGE, GNOMELOCALEDIR);  
+        gtk_set_locale();
+	bindtextdomain(PACKAGE, GNOMELOCALEDIR);
 	textdomain(PACKAGE);
 
 #ifdef HAVE_SOCKS5
 	SOCKSinit(argv[0]);
 #endif
 
+	srand( time( NULL ) );
+
 	our_info = g_new0( USER_INFO_STRUCT, 1 );
 	our_info->country = 1; /* USA */
 
 	toggles = g_new0( _toggles, 1 );
+	programs = g_new0( _programs, 1 );
 
 	MainData = g_new0( struct sokandlb, 1 );
 
 	WindowTitle = g_strdup( _("GnomeICU") );
 	configfilename = g_strdup( "/GnomeICU/" );
 
-	Current_Status = STATUS_OFFLINE;
+	toggles->applet = TRUE;
 
-	make_applet( argc, argv );
-	gnome_config_push_prefix( configfilename );
-
-	if( toggles->applet == FALSE )
-	{
-		cfnp = 0;
-		gnome_init_with_popt_table( PACKAGE, VERSION, argc, argv,
-		                            arguments, 0, NULL );
-		if( cfnp )
-		  {
-		    g_free( configfilename );
-		    configfilename = g_strdup_printf( "/GnomeICU_%s/", cfnp );
-		  }
+	/* We do this here now so we can call our init() function */
+	/* right off the bat.  Otherwise we run into many issues  */
+	for (cx = 0; cx < argc; cx++) {
+		if (!strcmp (argv[cx], _("-u")) && argc > cx + 1) {
+			g_free (configfilename);
+			configfilename = g_strdup_printf ("/GnomeICU_%s/", argv[cx + 1]);
+		}
+		if (!strcmp (argv[cx], _("-a")) || !strcmp (argv[cx], _("--noapplet")))
+			toggles->applet = FALSE;
 	}
 
-	Get_Unix_Config_Info();
+	Current_Status = STATUS_OFFLINE;
+
+	if (toggles->applet == FALSE) {
+		gnome_init_with_popt_table (PACKAGE, VERSION, argc, argv,
+		                            arguments, 0, NULL);
+		Get_Unix_Config_Info();
+	}
+	else {
+		applet_widget_init( PACKAGE, VERSION, argc, argv,
+		                    arguments, 0, NULL );
+		Get_Unix_Config_Info();
+		make_applet (argc, argv, arguments);
+	}
+
+	gnome_config_push_prefix( configfilename );
+
+	if( toggles->check_spell )
+	if (gtkspell_start(NULL, ispell_cmd) < 0)
+		fprintf(stderr, "Unable to start ispell; spellchecking disabled.\n");
+
+	/* These SIGPIPEs are irritating, catch them */
+	signal(SIGPIPE, broken_pipe);
 
 	if ( toggles->applet == FALSE )
 	{
 		client = gnome_master_client();
-		gtk_signal_connect( GTK_OBJECT( client ), "destroy",
+		gtk_signal_connect( GTK_OBJECT( client ), "die",
 		                    GTK_SIGNAL_FUNC( icq_quit ),
 		                    NULL );
 
@@ -433,10 +501,19 @@ int main( int argc, char *argv[] )
 	gtk_widget_push_visual(gdk_imlib_get_visual());
 	gtk_widget_push_colormap(gdk_imlib_get_colormap());
 
-	app = gnome_app_new ( "Gnomeicu", WindowTitle );
+#ifdef GNOME_ICON
+	{
+		gchar *icon_path = make_file_path_theme( "gnomeicu-online.png", "Default" );
+		gnome_window_icon_set_default_from_file( icon_path );
+		g_free( icon_path );
+	}
+#endif
+
+   	app = gnome_app_new ( "GnomeICU", WindowTitle );
+        gtk_window_set_wmclass (GTK_WINDOW (app), WindowTitle, "GnomeICU");
 	gtk_object_set_data (GTK_OBJECT (app), "app", app);
 	gtk_window_set_default_size (GTK_WINDOW (app), WindowWidth, WindowHeight );
-	gtk_window_set_policy (GTK_WINDOW (app), TRUE, TRUE, FALSE);
+	gtk_window_set_policy (GTK_WINDOW (app), FALSE, TRUE, FALSE);
 
 	MainData->window = app;
 
@@ -491,6 +568,14 @@ int main( int argc, char *argv[] )
 	gtk_clist_set_column_width (GTK_CLIST (MainData->lb_userwin_online), 1, 10);
 	gtk_clist_set_selection_mode (GTK_CLIST (MainData->lb_userwin_online), GTK_SELECTION_BROWSE);
 	gtk_clist_column_titles_hide (GTK_CLIST (MainData->lb_userwin_online));
+	gtk_clist_set_sort_column(GTK_CLIST(MainData->lb_userwin_online), 1);
+
+	/* FIXME: this method of sorting breaks everything
+	 * (mostly kontakt->lb_index). Sorting should be done in
+	 * Contacts/showlist.c::OnlineList level. Or connect to some signal
+	 * here? */
+	if ( toggles->sort_contacts )
+		gtk_clist_set_auto_sort(GTK_CLIST(MainData->lb_userwin_online), TRUE);
 
 	MainData->lb_userwin = MainData->lb_userwin_online;
 
@@ -511,21 +596,24 @@ int main( int argc, char *argv[] )
 
 	init_contact_list_drag_drop( MainData->lb_userwin_online );
 
-	label5 = gtk_label_new (_("label5"));
+	label5 = gtk_label_new ("label5");
 	gtk_widget_ref (label5);
 	gtk_object_set_data_full (GTK_OBJECT (app), "label5", label5,
 	                          (GtkDestroyNotify) gtk_widget_unref);
 	gtk_widget_show (label5);
 	gtk_clist_set_column_widget (GTK_CLIST (MainData->lb_userwin_online), 0, label5);
 
-	label6 = gtk_label_new (_("label6"));
+	label6 = gtk_label_new ("label6");
 	gtk_widget_ref (label6);
 	gtk_object_set_data_full (GTK_OBJECT (app), "label6", label6,
 	                          (GtkDestroyNotify) gtk_widget_unref);
 	gtk_widget_show (label6);
 	gtk_clist_set_column_widget (GTK_CLIST (MainData->lb_userwin_online), 1, label6);
 
-	MainData->head_online = gtk_label_new (_("Online"));
+        /* The space at the end is because this version of Online is for
+         * the notebook tab, and adjectives may be inflected in plural,
+         * so we need to have gettext separate this one out. */
+	MainData->head_online = gtk_label_new (_("Online "));
 	gtk_widget_ref (MainData->head_online);
 	gtk_object_set_data_full (GTK_OBJECT (app), "head_online", MainData->head_online,
 	                          (GtkDestroyNotify) gtk_widget_unref);
@@ -571,7 +659,7 @@ int main( int argc, char *argv[] )
 
 	if( toggles->applet == FALSE )
 	{
-		gtk_signal_connect( GTK_OBJECT( app ), "destroy",
+		gtk_signal_connect( GTK_OBJECT( app ), "delete_event",
 		                    GTK_SIGNAL_FUNC( icq_quit ), NULL );
 		style = gtk_widget_get_style( MainData->lb_userwin );
 
@@ -581,24 +669,30 @@ int main( int argc, char *argv[] )
 		gtk_signal_connect( GTK_OBJECT( app ), "delete_event",
 		                    GTK_SIGNAL_FUNC( applet_hide_main ), 0 );
 
+	gtk_signal_connect( GTK_OBJECT( app ), "size-allocate",
+			    GTK_SIGNAL_FUNC( window_save_size ), NULL );
+
 	if( toggles->show_window )
 		gtk_widget_show_all( app );
 
-	label3 = gtk_label_new (_("label3"));
+	label3 = gtk_label_new ("label3");
 	gtk_widget_ref (label3);
 	gtk_object_set_data_full (GTK_OBJECT (app), "label3", label3,
 	                          (GtkDestroyNotify) gtk_widget_unref);
 	gtk_widget_show (label3);
 	gtk_clist_set_column_widget (GTK_CLIST (MainData->lb_userwin_offline), 0, label3);
 
-	label4 = gtk_label_new (_("label4"));
+	label4 = gtk_label_new ("label4");
 	gtk_widget_ref (label4);
 	gtk_object_set_data_full (GTK_OBJECT (app), "label4", label4,
 	                          (GtkDestroyNotify) gtk_widget_unref);
 	gtk_widget_show (label4);
 	gtk_clist_set_column_widget (GTK_CLIST (MainData->lb_userwin_offline), 1, label4);
 
-	MainData->head_offline = gtk_label_new (_("Offline"));
+        /* The space at the end is because this version of Offline is for
+         * the notebook tab, and adjectives may be inflected in plural,
+         * so we need to have gettext separate this one out. */
+	MainData->head_offline = gtk_label_new (_("Offline "));
 	gtk_widget_ref (MainData->head_offline);
 	gtk_object_set_data_full (GTK_OBJECT (app), "head_offline", MainData->head_offline,
 	                          (GtkDestroyNotify) gtk_widget_unref);
@@ -642,14 +736,14 @@ int main( int argc, char *argv[] )
 
 	init_contact_list_drag_drop( MainData->lb_userwin_notinlist );
 
-	label7 = gtk_label_new (_("label7"));
+	label7 = gtk_label_new ("label7");
 	gtk_widget_ref (label7);
 	gtk_object_set_data_full (GTK_OBJECT (app), "label7", label7,
 	                          (GtkDestroyNotify) gtk_widget_unref);
 	gtk_widget_show (label7);
 	gtk_clist_set_column_widget (GTK_CLIST (MainData->lb_userwin_notinlist), 0, label7);
 
-	label8 = gtk_label_new (_("label8"));
+	label8 = gtk_label_new ("label8");
 	gtk_widget_ref (label8);
 	gtk_object_set_data_full (GTK_OBJECT (app), "label8", label8,
 	                          (GtkDestroyNotify) gtk_widget_unref);
@@ -682,7 +776,7 @@ int main( int argc, char *argv[] )
 	                  (GtkAttachOptions) (GTK_SHRINK | GTK_FILL),
 	                  (GtkAttachOptions) (0), 0, 0);
 
-	gtk_signal_connect( GTK_OBJECT( status_button ), "clicked",
+	gtk_signal_connect( GTK_OBJECT( status_button ), "button_press_event",
 	                    GTK_SIGNAL_FUNC( popup_status_menu ), NULL );
 
 	hbox1 = gtk_hbox_new( FALSE, 0 );
@@ -695,21 +789,12 @@ int main( int argc, char *argv[] )
 	gtk_widget_ref (eyes_still);
 	gtk_object_set_data_full (GTK_OBJECT (app), "eyes_still", eyes_still,
 	                          (GtkDestroyNotify) gtk_widget_unref);
+#ifndef NO_SCARY_EYEBALLS
 	gtk_widget_show (eyes_still);
+#endif
 	gtk_box_pack_end( GTK_BOX( hbox1 ), eyes_still, FALSE, FALSE, 0 );
 
-	eyes_animated = gnome_animator_new_with_size( 29, 14 );
-
-	str = gnome_pixmap_file( "gnomeicu/animeyes.png" );
-
-	if( str == NULL )
-		str = gnome_pixmap_file( "animeyes.png" );
-
-	gnome_animator_append_frames_from_file( GNOME_ANIMATOR( eyes_animated ),
-	                                        str, 0, 0, 100, 29 );
-	g_free( str );
-
-	gtk_box_pack_end( GTK_BOX( hbox1 ), eyes_animated, FALSE, FALSE, 0 );
+        create_animator( hbox1 );
 
 	appbar1 = gnome_appbar_new (FALSE, FALSE, GNOME_PREFERENCES_NEVER);
 	gtk_widget_ref (appbar1);
@@ -729,7 +814,7 @@ int main( int argc, char *argv[] )
 		gtk_signal_connect( GTK_OBJECT (app), "expose_event",
 				    GTK_SIGNAL_FUNC (start_autohide_timeout_cb), NULL );
 
-		autohide_cbs_are_connected = TRUE;	
+		autohide_cbs_are_connected = TRUE;
 	}
 
 	init_colors();
@@ -738,9 +823,10 @@ int main( int argc, char *argv[] )
 
 /*** New log window ***/
 	log_window = gnome_dialog_new (_("GnomeICU: Connection History"), NULL);
+	gtk_window_set_wmclass (GTK_WINDOW (log_window), "Connection_History", "GnomeICU");
 	gtk_object_set_data (GTK_OBJECT (log_window), "log_window", log_window);
 	gtk_window_set_default_size (GTK_WINDOW (log_window), 500, 300);
-	gtk_window_set_policy (GTK_WINDOW (log_window), TRUE, TRUE, FALSE);
+	gtk_window_set_policy (GTK_WINDOW (log_window), FALSE, TRUE, FALSE);
 
 	dialog_vbox1 = GNOME_DIALOG (log_window)->vbox;
 	gtk_object_set_data (GTK_OBJECT (log_window), "dialog_vbox1", dialog_vbox1);
@@ -821,12 +907,14 @@ int main( int argc, char *argv[] )
 	                           GTK_OBJECT( log_window ));
 /*** End new log window ***/
 
+	/* FIXME: is this assignment needed? it is done several times in other
+	 * places */
 	contact = Contacts;
 
 	while( contact != NULL )
 	{
-		((CONTACT_PTR)contact->data)->icon_p = icon_offline_pixmap;
-		((CONTACT_PTR)contact->data)->icon_b = icon_offline_bitmap;
+		kontakt->icon_p = icon_offline_pixmap;
+		kontakt->icon_b = icon_offline_bitmap;
 		contact = contact->next;
 	}
 
@@ -894,9 +982,16 @@ int main( int argc, char *argv[] )
 			Save_RC();
 		}
 
-		Login( passwd, our_ip, our_port );
+		if ( toggles->connect_startup )
+		{
+			Login( passwd, our_ip, our_port );
+		}
 	}
 
+	/* bootup unix socket */
+	userserver_init();
+        gnome_win_hints_init();
+
 	if( toggles->applet )
 	{
 		applet_update( STATUS_OFFLINE, NULL );
@@ -905,6 +1000,9 @@ int main( int argc, char *argv[] )
 	else
 		gtk_main();
 
+	/* shutdown unix socket */
+	userserver_kill();
+
 	Quit_ICQ();
 	return 0;
 }
@@ -916,26 +1014,17 @@ source_drag_data_get  (GtkWidget	*widget, GdkDragContext *context,
 {
 	gchar *str;
 	GSList *contact;
-	GtkCList *clist = GTK_CLIST( MainData->lb_userwin );
-
-	contact = Contacts;
-
-	while( contact != NULL )
-	{
-		if( GPOINTER_TO_INT( gtk_clist_get_row_data( GTK_CLIST( clist ), GPOINTER_TO_INT( clist->selection->data ) ) ) ==
-		    ((CONTACT_PTR)contact->data)->uin )
-			break;
-		contact = contact->next;
-	}
 
+	contact = get_contact_from_clist();
 	if( contact == NULL )
 		return;
 
-	str = g_strdup_printf( "%d\n%s", ((CONTACT_PTR)contact->data)->uin,
-	                       ((CONTACT_PTR)contact->data)->nick );
+	str = g_strdup_printf( "%d\n%s", kontakt->uin,
+	                       kontakt->nick );
 	gtk_selection_data_set( selection_data,
 	                        selection_data->target,
 	                        8, str, strlen( str ) );
+	g_free( str );
 }
 
 void change_userwin( GtkWidget *widget, GtkNotebookPage *page, int num,
