@@ -25,6 +25,8 @@ extern gchar *Away_Message;
 extern gboolean Done_Login;
 extern gint connecting;
 
+extern _programs *programs;
+
 extern gboolean search_in_progress;
 extern GtkWidget *log_list;
 extern GtkWidget *log_window;
@@ -84,6 +86,7 @@ extern GtkWidget *eyes_still;
 
 void create_tcp_line( void );
 void ready_set( void );
+void create_animator( GtkWidget *hbox_parent );
 void toggle_log_window( GtkWidget *widget, gpointer data );
 void init( void );
 
